/*
 Log to SD card Humidity and Temp Sensors sent
 from XBEE on Ardunio

 USES xBEE LIB CREATED BY ANDY INSTEAD OF STEVE'S SERIAL LIBRARY
 FIRST TRY LEAVE TINY LIB AND COMMENT OUT PUTCHAR() BECAUSE THAT WORKS
 SO LETS SEE IF IT STILL DOES WITH ANDY'S LIB SERIAL CODE.

 EACH RUN OF THE PROGRAM WILL MAKE A PASS THRU THE STATE MACHINE OF
 GETTING A NEW FILE NAME, WAITING TO LOG, LOGGING, AND STOP LOGGING

*/

#include "simpletools.h"                      // Include simple tools
#include "xb.h"                         // Use Andy's XBEE LIb  <===

// SD Card pin connections
int DO = 22, CLK = 23, DI = 24, CS = 25;
const int btnpin = 0;
int RX = 5, TX = 4;                           // XBEE connected pins

// The initial filename for logging, 99 possible names will be created
// Restarting logging increments to the next number ex XBEEHUM01,XBEEHUM02 etc
char logfilename[] = "XBHUM00.CSV";
FILE* logfile;

// Use fread() to determine if the file already exists by trying to open it for read
_Bool file_exists(char *fn) {
  FILE* fp = fopen(fn,"r");
  pause(50);
  if(fp != NULL) fclose(fp);
  return (fp != NULL);
}

// Get the next filename to log to up to 99 files
FILE* Getnextfile() {
FILE* fp = NULL;

  for(int xx = 0;xx < 100;xx++) {
    logfilename[5] = xx/10 + '0';
    logfilename[6] = xx%10 + '0';
    printf(logfilename);
    printf("\n");
    if(!file_exists(logfilename)) {
      fp = fopen(logfilename,"w");
      pause(50);
      // Bail out of loop if we have a new file name
      break;
    }
  } // end for
  //We either have FILE* for our file name or we used up all 100 possible names 00..99
  return fp;
}

//Return a 1 if we release the button after a press
int GetButton( int pin ) {
static int rawprev = 1;                        // Previous value used in debouncing
int rawnew;                                    // New value used in debouncing
const int tsame = 25;                          // 50ms if we get called every 2ms
static int tdebounce = 0;                      // Counter used in debouncing
static int prev = 1;                           //Button has pull up and will be normally high=1
int new;                                       //Recent read of pin value Button is connected to
int change;                                    //A high=1 if a change from last read of pin state

  rawnew = input(pin);
  if(rawnew != rawprev) {
    // It changed so start debounce
    tdebounce = 0;
    rawprev = rawnew;                         //Save value
    return(0); 
  }
  else {
    //See if it has been the same for debounce time
    if(tdebounce >= tsame) {
     // We have stable value use it
     new = rawnew;
     change = new ^ prev;                        // new xor prev is a 1 if state changed
     prev = new;                                 // save present state
     rawprev = rawnew;
     tdebounce = 0;                              // Clear the counter
     return ( change & new );                    // change and new is a 1 if this is a release of button
    }
    else {
     tdebounce++;                                // Inc debounce counter
     rawprev = rawnew;
     return(0);
    }
  }
    
}

// Clear our line buffer to all NULLS
void ClearLine(char *s) {
 // Hard coded for our aline array, 
 for(int i=0;i<80;i++) {
  s[i] = 0;
 }
}


int main()                                    // Main function
{
static int state = 0;
int btnrelease;
char aline[80];                               //A line of text from the XBEE
int lineix;                                   //Index to where to store next XBEE rec char
int ch;                                       // Characters from XBEE
unsigned int linecount;                       // Add linecount to each line received from XBEE

  // Add startup code here.
  pause(1000); // Wait for terminal to open
  linecount = 0;

  sd_mount(DO, CLK, DI, CS);
  xb_start(RX,TX,0,9600);                    // Full Duplex serial to XBEE  <===
//  putchar(CLS);                              // Clear terminal screen
  xb_rxflush();                        // Clear Serial Buffer  <===
  pause(100);

  printf("Trying to get a new file\n");
  logfile = Getnextfile();
  if(logfile != NULL) {
   printf("Got %s\n",logfilename);
  }
  else {
   printf("Unable to find a new file\n");
   exit;                                     // break out of main() ??
  }

  while(1)                                    // Repeat indefinitely
  {
    // Add main loop code here.

    // The only input to the state machine is button release which increments
    // to the next state with each release with the last state waiting for 
    // power down
    
    // Check the Button if not in init state 0
    if(state > 0) { 
      btnrelease = GetButton(btnpin);
      if(btnrelease) {
       high(26);
      }
      else {
       low(26);
      }
    }

    // Execute state machine based on state and if a button was released
    switch ( state ) {
    case 0: // Init logging 
       printf("Press and release button to start logging\n");
       // Clear our line buffer to all NULL's
       ClearLine(aline);
       lineix = 0;
       state = 1;       // Move to next state
       break;
    case 1: // Wait for button release to start logging
       if(btnrelease) {
         state = 2;
         printf("Started logging\n");
       }
       break;           
    case 2: // Log text lines sent by the XBEE of sensor data to the SD card
       ch = xb_rxcheck();             //  <===
       if( ch != -1) {
         if(ch == 13) {
           aline[lineix] = 0;        //Add a NULL to end of line
           fprintf(logfile,"%u,%s\n",linecount++,aline);   // Write the line to the SD card
           pause(500);                      // Give it time to do the write
           printf("%s\n",aline);      // Show it on terminal so we can see what is happening
           lineix = 0;                // Get ready for the next line from XBEE
           ClearLine(aline);
           xb_rxflush();              // <===
         }
         else {
          aline[lineix] = ch;       // Add next char to line at index
          lineix++;                 // Increment index
          if(lineix > 78){          // Limit index to prevent runnning wild
            lineix = 78;
          }
         }
       }
       if(btnrelease) {
         state = 3;
       }
       break;
    case 3: // Stop Logging
       fclose(logfile);                  // Close the logging file
       printf("Closed Log File => %s\n",logfilename);
       state = 4;                   // Go to do nohing state
       break;
    case 4: // Spin here until power down 
       break;
    } // End case
    pause(2);                      // Pace the getting of button states at 2ms       
  } // End while  
} // End main()
